indexing
	description: "Objects that print static program point information in .decls format %
		%and produce instructions to print program point state in .dtrace format %
		%This implementation uses the new format"
	author: "Markus Pilman"
	date: "$Date$"
	revision: "$Revision$"

class
	CI_NEW_DAIKON_PROGRAM_POINT_PRINTER

inherit
	CI_DAIKON_PROGRAM_POINT_PRINTER
	redefine
		output_caption,
		output_ppt_info,
		output_variable_info
	end

feature -- Output
	output_caption is
			-- Ouput general information about declarations into static output
		do
			static_output.put_string ((create {CI_FIELD}.make (version_field, version_value, 0)).out)
			static_output.new_line
		end

feature {NONE} -- Output
	output_ppt_info (ppt_name: STRING) is
			-- Print the prologue of a new program point declaration
		local
			parent: CI_FIELD
		do
			static_output.put_string ((create {CI_FIELD}.make (program_point_field, ppt_name, 0)).out)
			static_output.put_string ((create {CI_FIELD}.make (program_point_type_field, current_program_point.kind (ppt_types), 0)).out)
			current_program_point.clean_suppressions -- Clean before outputting
			from
				current_program_point.suppressions.start
			until
				current_program_point.suppressions.after
			loop
				create parent.make (parent_field, parent_relation_type + " " +
					program_point_name (current_program_point.suppressions.item.suppressor) + " " +
					current_program_point.suppressions.index.out, 0)
				static_output.put_string (parent.out)
				current_program_point.suppressions.forth
			end
		end

	output_variable_info (a_variable: CI_VARIABLE) is
			-- Static information about `a_variable' in Daikon format:
			-- its name, declared type, representation type and comparability information
		local
			parent_value: STRING
			parent: CI_FIELD
			renamed_variable: CI_VARIABLE
			query: CI_QUERY
			flags: STRING
		do
			-- Output variable name
			static_output.put_string ((create {CI_FIELD}.make (variable_field, a_variable.name, 1)).out)
			-- Output variable kind
			if a_variable.is_query then
				query ?= a_variable
				check query /= Void end
				static_output.put_string ((create {CI_FIELD}.make (variable_kind_field, query_kind_value + " " + query.unqualified_name, 2)).out)
				static_output.put_string ((create {CI_FIELD}.make (target_field, query.target.name, 2)).out)
			elseif a_variable.is_result then
				static_output.put_string ((create {CI_FIELD}.make (variable_kind_field, result_kind_value, 2)).out)
			else
				static_output.put_string ((create {CI_FIELD}.make (variable_kind_field, plain_kind_value, 2)).out)
			end
			-- Output declared type
			if a_variable.type.kind.is_printable then
				static_output.put_string ((create {CI_FIELD}.make (declared_type_field, a_variable.type.representation_type, 2)).out)
			else
				static_output.put_string ((create {CI_FIELD}.make (declared_type_field, a_variable.type.base_class.upper_name, 2)).out)
			end
			-- Output representation type
			static_output.put_string ((create {CI_FIELD}.make (representation_type_field, a_variable.type.representation_type, 2)).out)
			-- Output flags
			create flags.make_empty
			if a_variable.is_param then
				flags.append (is_param_value + " ")
			end
			if a_variable.is_non_null then
				flags.append (non_null_value + " ")
			end
			if not flags.is_empty then
				static_output.put_string ((create {CI_FIELD}.make (flags_field, flags, 2)).out)
			end
			-- Output comparability
			static_output.put_string ((create {CI_FIELD}.make (comparability_field, a_variable.comparability.out, 2)).out)
			-- Output suppressions
			from
				current_program_point.suppressions.start
			until
				current_program_point.suppressions.after
			loop
				renamed_variable := a_variable.renamed (current_program_point.suppressions.item)
				if renamed_variable /= Void and current_program_point.suppressions.item.suppressor.flattened_variables.has (renamed_variable) then
					parent_value := program_point_name (current_program_point.suppressions.item.suppressor) + " " +
						current_program_point.suppressions.index.out
					if not renamed_variable.name.is_equal (a_variable.name) then
						parent_value.append (" ")
						parent_value.append (renamed_variable.name)
					end
					create parent.make (parent_field, parent_value, 2)
					static_output.put_string (parent.out)
				end
				current_program_point.suppressions.forth
			end
			-- Output constant value
			if a_variable.constant /= Void then
				static_output.put_string ((create {CI_FIELD}.make (constant_field, constant_value (a_variable.constant), 2)).out)
			end
		end

feature {NONE} -- Constants
	version_field: STRING is "decl-version"
			-- Version field name

	version_value: STRING is "2.0"
			-- Version field value

	program_point_field: STRING is "ppt"
			-- Program point declaration field name

	parent_field: STRING is "parent"
			-- Parent declaration field name

	parent_relation_type: STRING is "parent"
			-- "Parent" type of suppression relation

	program_point_type_field: STRING is "ppt-type"
			-- Program point type field name

	variable_field: STRING is "variable"
			-- Variable declaration field name

	variable_kind_field: STRING is "var-kind"
			-- Variable kind field name

	plain_kind_value: STRING is "variable"
			-- Value of variable kind field denoting plain variable

	query_kind_value: STRING is "field"
			-- Value of variable kind field denoting query

	result_kind_value: STRING is "return"
			-- Value of variable kind field denoting Result

	target_field: STRING is "enclosing-var"
			-- Variable kind field name						

	declared_type_field: STRING is "dec-type"
			-- Declared type field name

	representation_type_field: STRING is "rep-type"
			-- Representation type field

	flags_field: STRING is "flags"
			-- Flags field name

	is_param_value: STRING is "is_param"
			-- Flag thet indicated that a variable is a formal argument

	non_null_value: STRING is "non_null"
			-- Flag thet indicated that a variable cannot take value 0 (Void)

	comparability_field: STRING is "comparability"
			-- Flags field name			

	constant_field: STRING is "constant"
			-- Constant field name

feature {NONE} -- Implementation
	ppt_types: CI_PPT_TYPE_FORMAT is
			-- Program point kind format for ppt-type tag
		once
			create Result
		end
end
