indexing
	description: "Objects that print static program point information and produce instructions to print program point state"
	author: "Nadia Polikarpova"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	CI_PROGRAM_POINT_PRINTER

inherit
	CI_RUNTIME_OUTPUT_METACLIENT

feature -- Processing
	process (a_program_point: CI_PROGRAM_POINT) is
			-- Print static information about `a_program_point' and build instructions that print its dynamic state
		require
			a_program_point_exists: a_program_point /= Void
			static_output_exists: static_output /= Void
			static_output_open: static_output.is_open_write or static_output.is_open_append
		deferred
		ensure
			state_printing_instructions_exists: state_printing_instructions /= Void
		end

feature -- Access
	state_printing_instructions: ET_COMPOUND
			-- State printing instructions for last processed program point

	extra_locals: ET_LOCAL_VARIABLE_LIST
			-- Additional locals that are used in `state_printing_instructions'

	static_output: FILE
			-- File for static information output

feature -- Setting
	set_static_output (an_output: like static_output) is
			-- Set `static_ouput' to `an_output'
		do
			static_output := an_output
		ensure
			static_output_set: static_output = an_output
		end

feature -- Output
	output_caption is
			-- Ouput general information about declarations into static output
		require
			static_output_set: static_output /= Void
		deferred
		end

	program_point_name (a_program_point: CI_PROGRAM_POINT): STRING is
			-- Program point name
		require
			a_program_point_exists: a_program_point /= Void
		deferred
		ensure
			result_exists: Result /= Void
		end

	input_program_point_name (a_name: STRING): STRING is
			-- Version of `a_name' in trace file, if `a_name' is a program point name from inferred assertions file; identity otherwise
		require
			a_name_exists: a_name /= Void
		deferred
		end
end
