indexing
	description: "Aggregative program points, representing class instances in all stable times %
		%(associated with class invariants)"
	author: "Nadia Polikarpova"
	date: "$Date$"
	revision: "$Revision$"

class
	CI_STABLE_TIME

inherit
	CI_PROGRAM_POINT
		redefine
			ensures_invariant,
			instrument
		end

create
	make

feature -- Access
	kind (a_visitor: CI_PROGRAM_POINT_KIND_FORMAT): STRING is
			-- Program point kind name
		do
			Result := a_visitor.stable_kind
		end

feature -- Status report
	ensures_invariant: BOOLEAN is
			-- Should invariant of instrumented class hold at program point?
		do
			Result := True
		end

feature {NONE} -- Variables
	add_variables is
			-- Add "Current" to variables list
		local
			var: CI_CURRENT
		do
			create var.make (Void, surrounding_class, universe)
			variables.put (var)
			add_constants
		end

feature -- Assertions
	instrument is
			-- Instrument program point by making `checked_assertions' call `state_printing_function'
		do
		end

	checked_assertions: ET_ASSERTIONS is
			-- Eiffel assertions, that are checked at this program point
		do
			Result := surrounding_class.invariants
		end

feature {NONE} -- Assertions
	create_assertions is
			-- Create empty `checked_assertions', if they don't exist
		local
			list: ET_INVARIANTS
		do
			if surrounding_class.invariants = Void then
				create list.make (surrounding_class)
				surrounding_class.set_invariants (list)
			end
		end

	process_original_assertions is
			-- Process original assertions according to settings while annotating this point
		local
			flat_inv: ET_INVARIANTS
		do
			if Settings.keep_original_contracts = Settings.remove_mode then
				checked_assertions.wipe_out
			elseif Settings.keep_original_contracts = Settings.flatten_mode then
				flat_inv := flat_invariants (surrounding_class, universe)
				checked_assertions.wipe_out
				prepend_assertions (checked_assertions, flat_inv)
			end
		end

	state_printing_function_name: ET_FEATURE_NAME is
			-- Name of `state_printing_function' for this program point
		do
			create {ET_IDENTIFIER} Result.make (surrounding_class.lower_name + "_inv_")
		end

	state_printing_function_formals: ET_FORMAL_ARGUMENT_LIST is
			-- Formal arguments of `state_printing_function'
		do
		end

	state_printing_function_actuals: ET_ACTUAL_ARGUMENT_LIST is
			-- Actual arguments of `state_printing_function' call inside instrumented `checked_assertions'
		do
		end
end
