indexing
	description: "Objects that evaluate types of Eiffel expressions %
		%Expression is supposed to be already checked for semantic validity"
	author: "Nadia Polikarpova"
	date: "$Date$"
	revision: "$Revision$"

class
	CI_EXPRESSION_TYPE_EVALUATOR

inherit
	ET_AST_ITERATOR
		rename
			make as make_with_universe
		export {NONE}
			make_with_universe
		redefine
			process_agent_implicit_current_target,	-- Not an expression
			process_agent_open_target,	-- Not an expression
			process_bit_constant,
			process_bracket_expression,
			process_c1_character_constant,
			process_c2_character_constant,
			process_c3_character_constant,
			process_call_agent,
			process_call_expression,
			process_convert_expression,
			process_convert_to_expression,
			process_create_expression,
			process_current,
			process_current_address,
			process_do_function_inline_agent,
			process_do_procedure_inline_agent,
			process_equality_expression,
			process_expression_address,
			process_external_function_inline_agent,
			process_external_procedure_inline_agent,
			process_false_constant,
			process_feature_address,
			process_hexadecimal_integer_constant,
			process_identifier,
			process_identifier_comma,
			process_infix_cast_expression,
			process_infix_expression,
			process_manifest_array,
			process_manifest_tuple,
			process_manifest_type,
			process_old_expression,
			process_once_function_inline_agent,
			process_once_manifest_string,
			process_once_procedure_inline_agent,
			process_parenthesized_expression,
			process_precursor_expression,
			process_prefix_expression,
			process_regular_integer_constant,
			process_regular_manifest_string,
			process_regular_real_constant,
			process_result,
			process_result_address,
			process_special_manifest_string,
			process_static_call_expression,
--			process_strip_expression, -- What is it???
			process_true_constant,
			process_underscored_integer_constant,
			process_underscored_real_constant,
			process_verbatim_string,
			process_void
		end

	CI_CODE_GENERATION
		export {NONE}
			all
		end

create
	make

feature -- Initialization
	make (a_closure: ET_STANDALONE_CLOSURE; a_context: like context; a_universe: like universe) is
			-- Create type evaluator for expression that appear in `a_closure' in `a_context' within `a_universe'
			-- `a_closure' may be void, if processed expressions are not supposed to contain formals and locals
		require
			a_context_exists: a_context /= Void
			a_universe_exists: a_universe /= Void
			implementation_checked: a_context.base_class (a_universe).implementation_checked
		do
			make_with_universe (a_universe)
			context := a_context

			create surrounding_closure.make
			if a_closure /= Void then
				surrounding_closure.put (a_closure)
			end
		ensure
			context_set: context = a_context
			universe_set: universe = a_universe
		end

feature {ET_AST_NODE} -- Processing
	process_agent_implicit_current_target (a_target: ET_AGENT_IMPLICIT_CURRENT_TARGET) is
			-- Process `a_target'
		do
			named_type := context.named_type (universe)
		end

	process_agent_open_target (a_target: ET_AGENT_OPEN_TARGET) is
			-- Process `a_target'
		do
			named_type := a_target.type.named_type (context, universe)
		end

	process_bit_constant (a_constant: ET_BIT_CONSTANT) is
			-- Process `a_constant'
		do
			create {ET_BIT_N} named_type.make (manifest_integer ((a_constant.literal.count - 1).out))
		end

	process_bracket_expression (an_expression: ET_BRACKET_EXPRESSION) is
			-- Process `an_expression'
		local
			old_context: ET_TYPE_CONTEXT
			bracket_function: ET_QUERY
		do
			an_expression.target.process (Current)
			if named_type /= Void then
				bracket_function := base_type.direct_base_class (universe).named_query (an_expression.name)
				if bracket_function /= Void then
					old_context := context
					context := new_context
					named_type := bracket_function.type.named_type (context, universe)
					context := old_context
				end
			end
		end

	process_c1_character_constant (a_constant: ET_C1_CHARACTER_CONSTANT) is
			-- Process `a_constant'.
		do
			process_character_constant (a_constant)
		end

	process_c2_character_constant (a_constant: ET_C2_CHARACTER_CONSTANT) is
			-- Process `a_constant'.
		do
			process_character_constant (a_constant)
		end

	process_c3_character_constant (a_constant: ET_C3_CHARACTER_CONSTANT) is
			-- Process `a_constant'.
		do
			process_character_constant (a_constant)
		end

	process_call_agent (an_agent: ET_CALL_AGENT) is
			-- Process `an_agent'
		local
			old_context: ET_TYPE_CONTEXT
			procedure: ET_PROCEDURE
			query: ET_QUERY
		do
			an_agent.target.process (Current)
			old_context := context
			context := new_context
			procedure := surrounding_class.named_procedure (an_agent.name)
			if procedure /= Void then
				process_agent (Void, procedure.arguments, an_agent.arguments, named_type, an_agent.target.is_open_operand)
			else
				query := surrounding_class.named_query (an_agent.name)
				check query /= Void end
				process_agent (query.type.named_type (context, universe), query.arguments, an_agent.arguments,
					named_type, an_agent.target.is_open_operand)
			end
			context := old_context
		end

	process_call_expression (an_expression: ET_CALL_EXPRESSION) is
			-- Process `an_expression'
		local
			target: ET_EXPRESSION
			old_context: ET_TYPE_CONTEXT
		do
			old_context := context
			target := an_expression.target
			if target /= Void then
				target.process (Current)
				context := new_context
			end

			if context /= Void then
				an_expression.name.process (Current)
			end
			context := old_context
		end

	process_convert_expression (a_convert_expression: ET_CONVERT_EXPRESSION) is
			-- Process `a_convert_expression'.
		do
			named_type := current_target_type
		end

	process_convert_to_expression (a_convert_expression: ET_CONVERT_TO_EXPRESSION) is
			-- Process `a_convert_expression'.
		do
			named_type := current_target_type
		end

	process_create_expression (an_expression: ET_CREATE_EXPRESSION) is
			-- Process `an_expression'
		do
			named_type := an_expression.creation_type.type.named_type (context, universe)
		end

	process_current (a_keyword: ET_CURRENT) is
			-- Process `a_keyword'
		do
			named_type := context.named_type (universe)
		end

	process_current_address (an_expression: ET_CURRENT_ADDRESS) is
			-- Process `an_expression'
		do
			an_expression.current_keyword.process (Current)
			check named_type /= Void end
			process_address_expression (named_type)
		end

	process_do_function_inline_agent (an_agent: ET_DO_FUNCTION_INLINE_AGENT) is
			-- Process `an_agent'
		do
			process_function_inline_agent (an_agent)
		end

	process_do_procedure_inline_agent (an_agent: ET_DO_PROCEDURE_INLINE_AGENT) is
			-- Process `an_agent'
		do
			process_procedure_inline_agent (an_agent)
		end

	process_equality_expression (an_expression: ET_EQUALITY_EXPRESSION) is
			-- Process `an_expression'
		do
			named_type := type_from_nongeneric_class (universe.boolean_class)
		end

	process_expression_address (an_expression: ET_EXPRESSION_ADDRESS) is
			-- Process `an_expression'
		do
			an_expression.expression.process (Current)
			if named_type /= Void then
				process_address_expression (named_type)
			end
		end

	process_external_function_inline_agent (an_agent: ET_EXTERNAL_FUNCTION_INLINE_AGENT) is
			-- Process `an_agent'
		do
			process_function_inline_agent (an_agent)
		end

	process_external_procedure_inline_agent (an_agent: ET_EXTERNAL_PROCEDURE_INLINE_AGENT) is
			-- Process `an_agent'
		do
			process_procedure_inline_agent (an_agent)
		end

	process_false_constant (a_constant: ET_FALSE_CONSTANT) is
			-- Process `a_constant'.
		do
			process_boolean_constant (a_constant)
		end

	process_feature_address (an_expression: ET_FEATURE_ADDRESS) is
			-- Process `an_expression'.
		do
			an_expression.name.process (Current)
			if named_type /= Void then
				process_address_expression (named_type)
			end
		end

	process_hexadecimal_integer_constant (a_constant: ET_HEXADECIMAL_INTEGER_CONSTANT) is
			-- Process `a_constant'
		do
			process_integer_constant (a_constant)
		end

	process_identifier (an_identifier: ET_IDENTIFIER) is
			-- Process `an_identifier'
		local
			query: ET_QUERY
			arg: ET_FORMAL_ARGUMENT
			local_var: ET_LOCAL_VARIABLE
		do
			query := surrounding_class.named_query (an_identifier)
			if query /= Void then
				named_type := query.type.named_type (context, universe)
			else
				if surrounding_closure /= Void and then surrounding_closure.arguments /= Void then
					arg := formal_by_name (surrounding_closure.arguments, an_identifier)
				end
				if arg /= Void then
					named_type := arg.type.named_type (context, universe)
				else
					if surrounding_closure /= Void and then surrounding_closure.locals /= Void then
						local_var := local_by_name (surrounding_closure.locals, an_identifier)
					end
					if local_var /= Void then
						named_type := local_var.type.named_type (context, universe)
					end
				end
			end
		end

	process_identifier_comma (an_identifier: ET_IDENTIFIER_COMMA) is
			-- Process `an_identifier'
		do
			process_identifier (an_identifier.identifier)
		end

	process_infix_cast_expression (an_expression: ET_INFIX_CAST_EXPRESSION) is
			-- Process `an_expression'
		do
			named_type := an_expression.type.named_type (context, universe)
		end

	process_infix_expression (an_expression: ET_INFIX_EXPRESSION) is
			-- Process `an_expression'
		local
			old_context: ET_TYPE_CONTEXT
			infix_function: ET_QUERY
		do
			an_expression.left.process (Current)
			if named_type /= Void then
				infix_function := base_type.direct_base_class (universe).named_query (an_expression.name)
				if infix_function /= Void then
					old_context := context
					context := new_context
					named_type := infix_function.type.named_type (context, universe)
					context := old_context
				end
			end
		end

	process_manifest_array (an_array: ET_MANIFEST_ARRAY) is
			-- Process `an_array'
		local
			params: ET_ACTUAL_PARAMETER_LIST
		do
			if not an_array.is_empty then
				an_array.first.expression.process (Current)
				if named_type /= Void then
					create params.make_with_capacity (1)
					params.put_first (named_type)
					create {ET_GENERIC_CLASS_TYPE} named_type.make (Void,
						universe.array_class.name,
						params,
						universe.array_class)
				end
			end
		end

	process_manifest_tuple (a_tuple: ET_MANIFEST_TUPLE) is
			-- Process `a_tuple'
		local
			params: ET_ACTUAL_PARAMETER_LIST
			i: INTEGER
		do
			create params.make_with_capacity (a_tuple.count)
			from
				i := a_tuple.count
			until
				i < 1
			loop
				a_tuple.item (i).expression.process (Current)
				if named_type = Void then
					i := 1
				else
					params.put_first (named_type)
				end
				i := i - 1
			end
			if params.count = a_tuple.count then
				create {ET_GENERIC_CLASS_TYPE} named_type.make (Void,
					universe.tuple_class.name,
					params,
					universe.tuple_class)
			end
		end

	process_manifest_type (a_type: ET_MANIFEST_TYPE) is
			-- Process `a_type'
		local
			params: ET_ACTUAL_PARAMETER_LIST
		do
			create params.make_with_capacity (1)
			params.put_first (a_type.type)
			create {ET_GENERIC_CLASS_TYPE} named_type.make (Void,
				universe.type_class.name,
				params,
				universe.type_class)
		end

	process_old_expression (an_expression: ET_OLD_EXPRESSION) is
			-- Process `an_expression'
		do
			an_expression.expression.process (Current)
		end

	process_once_function_inline_agent (an_agent: ET_ONCE_FUNCTION_INLINE_AGENT) is
			-- Process `an_agent'
		do
			process_function_inline_agent (an_agent)
		end

	process_once_manifest_string (a_string: ET_ONCE_MANIFEST_STRING) is
			-- Process `a_string'
		do
			named_type := universe.string_type
		end

	process_once_procedure_inline_agent (an_agent: ET_ONCE_PROCEDURE_INLINE_AGENT) is
			-- Process `an_agent'
		do
			process_procedure_inline_agent (an_agent)
		end

	process_parenthesized_expression (an_expression: ET_PARENTHESIZED_EXPRESSION) is
			-- Process `an_expression'
		do
			an_expression.expression.process (Current)
		end

	process_precursor_expression (an_expression: ET_PRECURSOR_EXPRESSION) is
			-- Process `an_expression'
		local
			old_context: ET_TYPE_CONTEXT
			surrounding_function: ET_FUNCTION
		do
			surrounding_function ?= surrounding_closure.item
			check
				only_in_function: surrounding_function /= Void
				function_has_precursor: surrounding_function.first_precursor /= Void
			end
			old_context := context
			-- TODO: think of context...
			named_type := surrounding_function.first_precursor.implementation_class
			context := new_context
			named_type := surrounding_function.first_precursor.type.named_type (context, universe)
			context := old_context
		end

	process_prefix_expression (an_expression: ET_PREFIX_EXPRESSION) is
			-- Process `an_expression'.
		local
			old_context: ET_TYPE_CONTEXT
			prefix_function: ET_QUERY
		do
			an_expression.expression.process (Current)
			if named_type /= Void then
				prefix_function := base_type.direct_base_class (universe).named_query (an_expression.name)
				if prefix_function /= Void then
					old_context := context
					context := new_context
					named_type := prefix_function.type.named_type (context, universe)
					context := old_context
				end
			end
		end

	process_regular_integer_constant (a_constant: ET_REGULAR_INTEGER_CONSTANT) is
			-- Process `a_constant'
		do
			process_integer_constant (a_constant)
		end

	process_regular_manifest_string (a_string: ET_REGULAR_MANIFEST_STRING) is
			-- Process `a_string'
		do
			process_manifest_string (a_string)
		end

	process_regular_real_constant (a_constant: ET_REGULAR_REAL_CONSTANT) is
			-- Process `a_constant'.
		do
			process_real_constant (a_constant)
		end

	process_result (a_keyword: ET_RESULT) is
			-- Process `a_keyword'
		do
			named_type := surrounding_closure.type.named_type (context, universe)
		end

	process_result_address (an_expression: ET_RESULT_ADDRESS) is
			-- Process `an_expression'
		do
			an_expression.result_keyword.process (Current)
			if named_type /= Void then
				process_address_expression (named_type)
			end
		end

	process_special_manifest_string (a_string: ET_SPECIAL_MANIFEST_STRING) is
			-- Process `a_string'
		do
			process_manifest_string (a_string)
		end

	process_static_call_expression (an_expression: ET_STATIC_CALL_EXPRESSION) is
			-- Process `an_expression'
		local
			old_context: ET_TYPE_CONTEXT
		do
			old_context := context
			named_type := an_expression.type.named_type (context, universe)
			context := new_context
			check context /= Void end
			an_expression.name.process (Current)
			context := old_context
		end

	process_true_constant (a_constant: ET_TRUE_CONSTANT) is
			-- Process `a_constant'.
		do
			process_boolean_constant (a_constant)
		end

	process_underscored_integer_constant (a_constant: ET_UNDERSCORED_INTEGER_CONSTANT) is
			-- Process `a_constant'
		do
			process_integer_constant (a_constant)
		end

	process_underscored_real_constant (a_constant: ET_UNDERSCORED_REAL_CONSTANT) is
			-- Process `a_constant'
		do
			process_real_constant (a_constant)
		end

	process_verbatim_string (a_string: ET_VERBATIM_STRING) is
			-- Process `a_string'
		do
			process_manifest_string (a_string)
		end

	process_void (a_keyword: ET_VOID) is
			-- Process `a_keyword'
		do
			named_type := universe.none_type
		end

feature {NONE} -- Internal processing
	process_address_expression (a_type: ET_TYPE) is
			-- Process address expression with argument of type `a_type'
		local
			params: ET_ACTUAL_PARAMETER_LIST
		do
			create params.make_with_capacity (1)
			params.put_first (a_type)

			create {ET_GENERIC_CLASS_TYPE} named_type.make (Void,
				universe.typed_pointer_class.name,
				params,
				universe.typed_pointer_class)
		end

	process_boolean_constant (a_constant: ET_BOOLEAN_CONSTANT) is
			-- Process `a_constant'
		do
			named_type := type_from_nongeneric_class (universe.boolean_class)
		end

	process_character_constant (a_constant: ET_CHARACTER_CONSTANT) is
			-- Process `a_constant'
		do
			if a_constant.type /= Void then
				named_type := a_constant.type
			elseif a_constant.cast_type /= Void then
				named_type := a_constant.cast_type.type.named_type (context, universe)
			else
				named_type := universe.character_type
			end
		end

	process_integer_constant (a_constant: ET_INTEGER_CONSTANT) is
			-- Process `a_constant'
		do
			if a_constant.type /= Void then
				named_type := a_constant.type
			elseif a_constant.cast_type /= Void then
				named_type := a_constant.cast_type.type.named_type (context, universe)
			else
				named_type := universe.integer_type
			end
		end

	process_manifest_string (a_string: ET_MANIFEST_STRING) is
			-- Process `a_string'
		do
			if a_string.type /= Void then
				named_type := a_string.type
			elseif a_string.cast_type /= Void then
				named_type :=a_string.cast_type.type.named_type (context, universe)
			else
				named_type := universe.string_type
			end

		end

	process_real_constant (a_constant: ET_REAL_CONSTANT) is
			-- Process `a_constant'
		do
			if a_constant.type /= Void then
				named_type := a_constant.type
			elseif a_constant.cast_type /= Void then
				named_type := a_constant.cast_type.type.named_type (context, universe)
			else
				named_type := universe.real_type
			end
		end

	process_function_inline_agent (an_agent: ET_FUNCTION_INLINE_AGENT) is
			-- Process `an_agent'
		do
			process_agent (an_agent.type, an_agent.formal_arguments, an_agent.actual_arguments,
				context.named_type (universe), false)
		end

	process_procedure_inline_agent (an_agent: ET_PROCEDURE_INLINE_AGENT) is
			-- Process `an_agent'
		do
			process_agent (Void, an_agent.formal_arguments, an_agent.actual_arguments,
				context.named_type (universe), false)
		end

	process_agent (a_type: ET_TYPE; a_formals: ET_FORMAL_ARGUMENT_LIST; an_actuals: ET_AGENT_ARGUMENT_OPERANDS;
		a_target_type: ET_NAMED_TYPE; is_target_open: BOOLEAN) is
			-- Process agent of closure with result type `a_type' and formal arguments `a_formals',
			-- which has operands `an_actuals', target type `a_target_type' and target opennes is indicated by `is_target_open'
		require
			an_actuals_exists: an_actuals /= Void
		local
			expr: ET_EXPRESSION
			explicit_args: ET_AGENT_ARGUMENT_OPERAND_LIST
			implicit_args: ET_AGENT_IMPLICIT_OPEN_ARGUMENT_LIST
			res_class: ET_CLASS
			params, arg_params: ET_ACTUAL_PARAMETER_LIST
			i, formal_count: INTEGER
		do
			if a_type = Void then
				res_class := universe.procedure_class
				create params.make_with_capacity (2)
			elseif a_type.same_named_type (universe.boolean_class, context, context, universe) then
				res_class := universe.predicate_class
				create params.make_with_capacity (2)
			else
				res_class := universe.function_class
				create params.make_with_capacity (3)
				params.put_first (a_type.named_type (context, universe))
			end

			if a_formals /= Void then
				formal_count := a_formals.count
			end
			create arg_params.make_with_capacity (a_formals.count + 1)
			implicit_args ?= an_actuals
			if implicit_args /= Void then
				from
					i := formal_count
				until
					i < 1
				loop
					arg_params.put_first (a_formals.item (i).type.named_type (context, universe))
					i := i - 1
				end
			else
				explicit_args ?= an_actuals
				check explicit_args /= Void end
				from
					i := formal_count
				until
					i < 1
				loop
					expr ?= explicit_args.item (i)
					if expr = Void then
						-- Argument is open
						arg_params.force_first (a_formals.item (i).type.named_type (context, universe))
					end
					i := i - 1
				end
			end
			if is_target_open then
				arg_params.put_first (a_target_type)
			end
			params.put_first (create {ET_TUPLE_TYPE}.make (arg_params))

			params.put_first (a_target_type)
			create {ET_GENERIC_CLASS_TYPE} named_type.make (Void, res_class.name, params, res_class)
		end


feature -- Access
	named_type: ET_NAMED_TYPE
			-- Named type of last processed expression
			-- Void, if expression type can't be evaluated in current context

	base_type: ET_BASE_TYPE is
			-- Base type of last processed expression
			-- Void, if expression type can't be evaluated in current context			
		do
			if named_type /= Void then
				Result := named_type.base_type (context, universe)
			end
		end

	base_class: ET_CLASS is
			-- Base class of last processed expression
			-- Void, if expression type can't be evaluated in current context			
		do
			if named_type /= Void then
				Result := named_type.base_class (context, universe)
			end
		end

	new_context: ET_TYPE_CONTEXT is
			-- Context, produced by last processed expression
			-- Void, if expression type can't be evaluated in current context			
		do
			if named_type /= Void then
				Result := context.new_type_context (named_type)
			end
		end

	surrounding_closure: CI_CLOSURE_STACK
			-- Closure, in which processed expression appears

feature -- Setting
	set_current_target_type (a_type: ET_NAMED_TYPE) is
			-- Set `current_target_type' to `a_type'
		do
			current_target_type := a_type
		ensure
			current_target_type_set: current_target_type = a_type
		end

feature {NONE} -- Implementation
	context: ET_TYPE_CONTEXT
			-- Context, in which processed expression appears

	surrounding_class: ET_CLASS is
			-- Class, in which processed expression appears
		do
			Result := context.base_class (universe)
		end

	current_target_type: ET_NAMED_TYPE
			-- Type of target of currently processed expression
			-- (if it is assignment source or actual argument)

invariant
	context_exists: context /= Void
	surrounding_class_exists: surrounding_class /= Void
	base_type_exists_when_named_type: named_type /= Void implies base_type /= Void
	new_context_exists_when_named_type: named_type /= Void implies new_context /= Void
end
