indexing
	description: "Objects that qualify expressions in preconditions%
		% (add specified target to all unqualified calls and replace `Current' by target)"
	author: "Nadia Polikarpova"
	date: "$Date$"
	revision: "$Revision$"

class
	CI_PRECONDITION_QUALIFIER

inherit
	CI_AST_MODIFIER
		rename
			make as make_with_universe,
			modified_node as qualified_node
		export {NONE}
			make_with_universe
		redefine
			process_bit_feature,
			process_call_agent,
			process_call_expression,
			process_convert_function,
			process_current,
			process_current_address,
			process_do_function_inline_agent,
			process_do_procedure_inline_agent,
			process_external_function_inline_agent,
			process_external_procedure_inline_agent,
			process_feature_address,
			process_formal_parameter_type,
			process_identifier,
			process_like_current,
			process_like_feature,
			process_once_function_inline_agent,
			process_once_procedure_inline_agent,
			process_qualified_call,
			process_static_call_expression
		end

	CI_CODE_GENERATION
		export {NONE}
			all
		end
create
	make

feature -- Initialization
	make (a_target: like call_target; a_surrounding_routine: ET_ROUTINE; a_context: like context; a_universe: like universe) is
			-- Create qualifier that qualifies with `a_target' of base class `a_target_base_class' within `a_universe'
		require
			a_target_exists: a_target /= Void
			a_context: a_context /= Void
			a_universe_exists: a_universe /= Void
		do
			make_with_universe (a_universe)
			call_target := parenthesized_for_dot (a_target)
			context := a_context
			target_base_class := context.base_class (universe)
			create agent_id_processor.make (a_surrounding_routine, context.root_context.direct_base_class (universe), universe)
		ensure
			call_target_set: (call_target = a_target) or (call_target.is_equal (parenthesized (a_target)))
			context_set: context = a_context
			universe_set: universe = a_universe
		end

feature -- Processing
	qualified (an_expression: ET_EXPRESSION): ET_EXPRESSION is
			-- Qualifed form of `an_expression'
		require
			an_expression_exists: an_expression /= Void
		do
			an_expression.process (Current)
			Result ?= qualified_node
		ensure
			is_query: an_expression.is_deep_equal (old an_expression)
			result_exists: Result /= Void
		end

feature {ET_AST_NODE} -- Processing
	process_bit_feature (a_type: ET_BIT_FEATURE) is
			-- Process `a_type'.
		do
			-- Obsolete, so no processing is really done, though feaure name should be replaced with its value
			qualified_node := a_type
		end

	process_call_agent (an_expression: ET_CALL_AGENT) is
			-- Replace target, name and arguments with their qualified form
		local
			t: ET_AGENT_TARGET
			implicit_target: ET_AGENT_IMPLICIT_CURRENT_TARGET
			n: ET_QUALIFIED_FEATURE_NAME
			qualified_call: ET_CALL_EXPRESSION
			args: ET_AGENT_ARGUMENT_OPERAND_LIST
		do
			an_expression.target.process (Current)
			t ?= qualified_node
			check t /= Void end
			implicit_target ?= t
			if implicit_target = Void then
				after_dot := True
			end

			an_expression.qualified_name.process (Current)
			n ?= qualified_node
			if n = Void then
				-- Feature name was replaced by qualified call
				qualified_call ?= qualified_node
				check qualified_call /= Void end
				check implicit_target /= Void end
				n := qualified_call.name
				t := qualified_call.target
			end
			after_dot := False

			args ?= an_expression.arguments
			if args /= Void then
				args.process (Current)
				args ?= qualified_node
				check args /= Void end
			end

			create {ET_CALL_AGENT} qualified_node.make (t, n, args)
		end

	process_call_expression (an_expression: ET_CALL_EXPRESSION) is
			-- Replace target, name and arguments with their qualified form
		local
			t: ET_EXPRESSION
			n: ET_QUALIFIED_FEATURE_NAME
			qualified_call: ET_CALL_EXPRESSION
			args: ET_ACTUAL_ARGUMENT_LIST
		do
			t := an_expression.target
			if t /= Void then
				t.process (Current)
				t ?= qualified_node
				check t /= Void end
				after_dot := True
			end

			an_expression.qualified_name.process (Current)
			n ?= qualified_node
			if n = Void then
				-- Feature name was replaced by qualified call
				qualified_call ?= qualified_node
				check qualified_call /= Void end
				check t = Void end
				n := qualified_call.name
				t := qualified_call.target
			end
			after_dot := False

			args := an_expression.arguments
			if args /= Void then
				args.process (Current)
				args ?= qualified_node
				check args /= Void end
			end

			create {ET_CALL_EXPRESSION} qualified_node.make (t, n, args)
		end

	process_convert_function (a_convert_function: ET_CONVERT_FUNCTION) is
			-- Do not replace name with call
		do
			qualified_node := a_convert_function
		end

	process_current (an_expression: ET_CURRENT) is
			-- Replace "Current" by `call_target'
		do
			qualified_node := call_target
		end

	process_current_address (an_expression: ET_CURRENT_ADDRESS) is
			-- Replace nested expression with its qualified form
		local
			id: ET_IDENTIFIER
		do
			-- Since there are no expression addresses in ISE and standard Eiffel, try to create identifier address
			id ?= call_target
			if id /= Void then
				create {ET_FEATURE_ADDRESS} qualified_node.make (id)
			else
				create {ET_EXPRESSION_ADDRESS} qualified_node.make (parenthesized (call_target))
			end
		end

	process_do_function_inline_agent (an_agent: ET_DO_FUNCTION_INLINE_AGENT) is
			-- Process `an_agent' with `agent_id_processor'
		local
			res: ET_DO_FUNCTION_INLINE_AGENT
		do
			inline_agent_depth := inline_agent_depth + 1
			if inline_agent_depth = 1 then
				an_agent.process (agent_id_processor)
				res ?= agent_id_processor.modified_node
				check res /= Void end
				Precursor (res)
			else
				Precursor (an_agent)
			end
			inline_agent_depth := inline_agent_depth - 1
			if inline_agent_depth = 0 then
				agent_id_processor.reset
			end
		end

	process_do_procedure_inline_agent (an_agent: ET_DO_PROCEDURE_INLINE_AGENT) is
			-- Process `an_agent' with `agent_id_processor'
		local
			res: ET_DO_PROCEDURE_INLINE_AGENT
		do
			inline_agent_depth := inline_agent_depth + 1
			if inline_agent_depth = 1 then
				an_agent.process (agent_id_processor)
				res ?= agent_id_processor.modified_node
				check res /= Void end
				Precursor (res)
			else
				Precursor (an_agent)
			end
			inline_agent_depth := inline_agent_depth - 1
			if inline_agent_depth = 0 then
				agent_id_processor.reset
			end
		end

	process_external_function_inline_agent (an_agent: ET_EXTERNAL_FUNCTION_INLINE_AGENT) is
			-- Process `an_agent' with `agent_id_processor'
		local
			res: ET_EXTERNAL_FUNCTION_INLINE_AGENT
		do
			inline_agent_depth := inline_agent_depth + 1
			if inline_agent_depth = 1 then
				an_agent.process (agent_id_processor)
				res ?= agent_id_processor.modified_node
				check res /= Void end
				Precursor (res)
			else
				Precursor (an_agent)
			end
			inline_agent_depth := inline_agent_depth - 1
			if inline_agent_depth = 0 then
				agent_id_processor.reset
			end
		end

	process_external_procedure_inline_agent (an_agent: ET_EXTERNAL_PROCEDURE_INLINE_AGENT) is
			-- Process `an_agent' with `agent_id_processor'
		local
			res: ET_EXTERNAL_PROCEDURE_INLINE_AGENT
		do
			inline_agent_depth := inline_agent_depth + 1
			if inline_agent_depth = 1 then
				an_agent.process (agent_id_processor)
				res ?= agent_id_processor.modified_node
				check res /= Void end
				Precursor (res)
			else
				Precursor (an_agent)
			end
			inline_agent_depth := inline_agent_depth - 1
			if inline_agent_depth = 0 then
				agent_id_processor.reset
			end
		end

	process_feature_address (an_expression: ET_FEATURE_ADDRESS) is
			-- Replace feature name with its qualified form
		local
			expr: ET_EXPRESSION
			id: ET_IDENTIFIER
		do
			Precursor (an_expression)
			-- Since there are no expression addresses in ISE and standard Eiffel, try to create identifier address
			expr ?= qualified_node
			id ?= qualified_node
			check expr /= Void end
			if id /= Void then
				create {ET_FEATURE_ADDRESS} qualified_node.make (id)
			else
				create {ET_EXPRESSION_ADDRESS} qualified_node.make (parenthesized (expr))
			end
		end

	process_formal_parameter_type (a_type: ET_FORMAL_PARAMETER_TYPE) is
			-- Replace formal type with named typed, that is associated with formal in client context
		do
			qualified_node := a_type.named_type (context, universe)
		end

	process_identifier (an_identifier: ET_IDENTIFIER) is
			-- If identifier is a query of `target_base_class', qualify it
		local
			qualified_call: ET_CALL_EXPRESSION
		do
			-- Note: Within an inline agent `an_identifier' may also be formal argument of routine,
			-- precondition of which is being process.
			-- However, since only zero argument queries take part in unfolding, it can't happen in present
			if not after_dot and (target_base_class.named_query (an_identifier) /= Void) then
				create qualified_call.make (call_target, an_identifier, Void)
				qualified_node := qualified_call
			else
				qualified_node := an_identifier
			end
		end

	process_like_current (a_type: ET_LIKE_CURRENT) is
			-- Replace type with named type, that is associated with `call_target' type in client context
		do
			qualified_node := a_type.named_type (context, universe)
		end

	process_like_feature (a_type: ET_LIKE_FEATURE) is
			-- Replace type with named type, that is associated with feature type in client context
		do
			qualified_node := a_type.named_type (context, universe)
		end

	process_once_function_inline_agent (an_agent: ET_ONCE_FUNCTION_INLINE_AGENT) is
			-- Process `an_agent' with `agent_id_processor'
		local
			res: ET_ONCE_FUNCTION_INLINE_AGENT
		do
			inline_agent_depth := inline_agent_depth + 1
			if inline_agent_depth = 1 then
				an_agent.process (agent_id_processor)
				res ?= agent_id_processor.modified_node
				check res /= Void end
				Precursor (res)
			else
				Precursor (an_agent)
			end
			inline_agent_depth := inline_agent_depth - 1
			if inline_agent_depth = 0 then
				agent_id_processor.reset
			end
		end

	process_once_procedure_inline_agent (an_agent: ET_ONCE_PROCEDURE_INLINE_AGENT) is
			-- Process `an_agent' with `agent_id_processor'
		local
			res: ET_ONCE_PROCEDURE_INLINE_AGENT
		do
			inline_agent_depth := inline_agent_depth + 1
			if inline_agent_depth = 1 then
				an_agent.process (agent_id_processor)
				res ?= agent_id_processor.modified_node
				check res /= Void end
				Precursor (res)
			else
				Precursor (an_agent)
			end
			inline_agent_depth := inline_agent_depth - 1
			if inline_agent_depth = 0 then
				agent_id_processor.reset
			end
		end

	process_qualified_call (a_call: ET_QUALIFIED_CALL) is
			-- Replaces actual arguments with their qualified form
		local
			args: ET_ACTUAL_ARGUMENT_LIST
		do
			-- Name is not processed: qualified call is always after dot
			args := a_call.arguments
			if args /= Void then
				args.process (Current)
				args ?= qualified_node
				check args /= Void end
			end
			create {ET_QUALIFIED_CALL} qualified_node.make (a_call.name, args)
		end

	process_static_call_expression (an_expression: ET_STATIC_CALL_EXPRESSION) is
			-- Replace target type and arguments with their qualified form
		local
			t: ET_TARGET_TYPE
			args: ET_ACTUAL_ARGUMENT_LIST
		do
			an_expression.static_type.process (Current)
			t ?= qualified_node
			check t /= Void end

			args := an_expression.arguments
			if args /= Void then
				args.process (Current)
				args ?= qualified_node
				check args /= Void end
			end

			create {ET_STATIC_CALL_EXPRESSION} qualified_node.make (t, an_expression.qualified_name, args)
		end

feature {NONE} -- Implementation
	call_target: ET_EXPRESSION
			-- Target with which calls are qualified

	target_base_class: ET_CLASS
			-- Base class of target

	after_dot: BOOLEAN
			-- Has dot already appeared in processed call expression?

	agent_id_processor: CI_INLINE_AGENT_IDENTIFIER_PROCESSOR

	inline_agent_depth: INTEGER

invariant
	call_target_exists: call_target /= Void
	target_base_class_exists: target_base_class /= Void
	context_exists: context /= Void
end
