indexing
	description: "Information about Eiffel classes that are printable (understood by Daikon)%
		% and about Daikon types assotiated with these classes"
	author: "Nadia Polikarpova"
	date: "$Date$"
	revision: "$Revision$"

class
	CI_PRINTABLE_CLASSES

inherit
	CI_SHARED_TOKEN_CONSTANTS

feature -- Eiffel classes
	printable_expanded_class_count: INTEGER is 15
			-- Number of printable expanded classes

	printable_class_count: INTEGER is 17
			-- Number of printable classes

	printable_expanded_classes: LIST [ET_CLASS_NAME] is
			-- Expanded classes, who's instances are printable variables (booleans, numbers, characters, pointers)
		once
			create {ARRAYED_LIST [ET_CLASS_NAME]} Result.make (printable_class_count)
			Result.extend (tokens.boolean_class_name)		-- BOOLEAN
			Result.extend (tokens.integer_8_class_name) 	-- INTEGER
			Result.extend (tokens.integer_16_class_name)
			Result.extend (tokens.integer_32_class_name)
			Result.extend (tokens.integer_64_class_name)
			Result.extend (tokens.natural_8_class_name)		-- NATURAL
			Result.extend (tokens.natural_16_class_name)
			Result.extend (tokens.natural_32_class_name)
			Result.extend (tokens.natural_64_class_name)
			Result.extend (tokens.character_8_class_name)	-- CHARACTER
			Result.extend (tokens.character_32_class_name)
			Result.extend (tokens.real_32_class_name)		-- REAL
			Result.extend (tokens.real_64_class_name)
			Result.extend (tokens.pointer_class_name)		-- POINTER
			Result.extend (tokens.typed_pointer_class_name)
			Result.compare_objects
		end

	printable_classes: LIST [ET_CLASS_NAME] is
			-- Classes, who's instances are printable variables (expanded printable variables + strings)
		once
			Result := printable_expanded_classes.twin
			Result.extend (tokens.string_8_class_name)		-- STRING
			Result.extend (tokens.string_32_class_name)
			Result.compare_objects
		end

	printable_class_refs: LIST [ET_CLASS_NAME] is
			-- For each <printable_expaneded_class> corresponding <printable_expanded_class>_REF
		once
			create {ARRAYED_LIST [ET_CLASS_NAME]} Result.make (printable_class_count)
			Result.extend (tokens.boolean_ref_class_name)
			Result.extend (tokens.integer_8_ref_class_name)
			Result.extend (tokens.integer_16_ref_class_name)
			Result.extend (tokens.integer_32_ref_class_name)
			Result.extend (tokens.integer_64_ref_class_name)
			Result.extend (tokens.natural_8_ref_class_name)
			Result.extend (tokens.natural_16_ref_class_name)
			Result.extend (tokens.natural_32_ref_class_name)
			Result.extend (tokens.natural_64_ref_class_name)
			Result.extend (tokens.character_8_ref_class_name)
			Result.extend (tokens.character_32_ref_class_name)
			Result.extend (tokens.real_32_ref_class_name)
			Result.extend (tokens.real_64_ref_class_name)
			Result.extend (tokens.pointer_ref_class_name)
			Result.extend (tokens.pointer_ref_class_name)
			Result.compare_objects
		end

	printable_class_indices: LIST [ET_IDENTIFIER] is
			-- Features from class `INTERNAL' that yield indices of printable classes, if exists (Void otherwise)
		once
			create {ARRAYED_LIST [ET_IDENTIFIER]} Result.make (printable_class_count)
			Result.extend (tokens.boolean_type_feature_name)
			Result.extend (tokens.integer_8_type_feature_name)
			Result.extend (tokens.integer_16_type_feature_name)
			Result.extend (tokens.integer_32_type_feature_name)
			Result.extend (tokens.integer_64_type_feature_name)
			Result.extend (tokens.natural_8_type_feature_name)
			Result.extend (tokens.natural_16_type_feature_name)
			Result.extend (tokens.natural_32_type_feature_name)
			Result.extend (tokens.natural_64_type_feature_name)
			Result.extend (tokens.character_8_type_feature_name)
			Result.extend (tokens.character_32_type_feature_name)
			Result.extend (tokens.real_32_type_feature_name)
			Result.extend (tokens.real_64_type_feature_name)
			Result.extend (tokens.pointer_type_feature_name)
			Result.extend (tokens.pointer_type_feature_name)
			Result.compare_objects
		end

feature -- Daikon types
	boolean_type: STRING is "boolean"
			-- Daikon type "boolean" (for boolean values)

	int_type: STRING is "int"
			-- Daikon type "int" (for integer values)

	double_type: STRING is "double"
			-- Daikon type "double" (for real values)

	string_type: STRING is "java.lang.String"
			-- Daikon type "String" (for character strings)

	hashcode_type: STRING is "hashcode"
			-- Daikon type "hashcode" (for unique object identities)

	daikon_type (a_printable_class: ET_CLASS_NAME): STRING is
			-- Daikon type that corresponds to `a_printable_class'
		require
			is_printable: printable_classes.has (a_printable_class)
		local
			i: INTEGER
		do
			i := printable_classes.index_of (a_printable_class, 1)
			if int_classes.has (i) then
				Result := int_type
			elseif double_classes.has (i) then
				Result := double_type
			elseif boolean_classes.has (i) then
				Result := boolean_type
			elseif string_classes.has (i) then
				Result := string_type
			elseif hashcode_classes.has (i) then
				Result := hashcode_type
			else
				check should_never_happen: False end
			end
		ensure
			result_exists: Result /= Void
		end

feature -- Status report
	is_printable (a_class: ET_CLASS): BOOLEAN is
			-- Is `a_class' one of `printable_classes'
		do
			Result := printable_classes.has (a_class.name)
		end

	is_printable_expanded (a_class: ET_CLASS): BOOLEAN is
			-- Is `a_class' one of `printable_expanded_classes'
		do
			Result := printable_expanded_classes.has (a_class.name)
		end

	is_string (a_class: ET_CLASS): BOOLEAN is
			-- Is `a_class' one of string classes?
		do
			Result := string_classes.has (printable_classes.index_of (a_class.name, 1))
		end

feature {NONE} -- Implementation
	boolean_classes: INTEGER_INTERVAL is
			-- Indices in `printable_classes' that correspond to classes with `boolean_type'
		once
			create Result.make (printable_classes.index_of (tokens.boolean_class_name, 1),
				printable_classes.index_of (tokens.boolean_class_name, 1))
		end

	int_classes: INTEGER_INTERVAL is
			-- Indices in `printable_classes' that correspond to classes with `int_type'
		once
			create Result.make (printable_classes.index_of (tokens.integer_8_class_name, 1),
				printable_classes.index_of (tokens.character_32_class_name, 1))
		end

	double_classes: INTEGER_INTERVAL is
			-- Indices in `printable_classes' that correspond to classes with `double_type'
		once
			create Result.make (printable_classes.index_of (tokens.real_32_class_name, 1),
				printable_classes.index_of (tokens.real_64_class_name, 1))
		end

	string_classes: INTEGER_INTERVAL is
			-- Indices in `printable_classes' that correspond to classes with `string_type'
		once
			create Result.make (printable_classes.index_of (tokens.string_8_class_name, 1),
				printable_classes.index_of (tokens.string_32_class_name, 1))
		end

	hashcode_classes: INTEGER_INTERVAL is
			-- Indices in `printable_classes' that correspond to classes with `hashcode_type'
		once
			create Result.make (printable_classes.index_of (tokens.pointer_class_name, 1),
				printable_classes.index_of (tokens.typed_pointer_class_name, 1))
		end

	character_classes: INTEGER_INTERVAL is
			-- Indices in `printable_classes' that correspond to variants of CHRACTER
		once
			create Result.make (printable_classes.index_of (tokens.character_8_class_name, 1),
				printable_classes.index_of (tokens.character_32_class_name, 1))
		end
invariant
	printable_expanded_class_not_empty: not printable_expanded_classes.is_empty
	printable_classes_not_empty: not printable_classes.is_empty
--	printable_classes_are_hashable: forall i = 1..printable_classes_count, a_universe: ET_UNIVERSE:
--		universe.eiffel_class (printable_classes [i]).ancestors.built implies
--		universe.eiffel_class (printable_classes [i]).has_ancestor (a_universe.hashable_class)
	correct_count: printable_class_count = printable_classes.count
	correct_expanded_count: printable_expanded_class_count = printable_expanded_classes.count
	expanded_have_references: printable_expanded_class_count = printable_class_refs.count
	expanded_have_indices: printable_expanded_class_count = printable_class_indices.count
	boolean_type_exists: boolean_type /= Void
	int_type_exists: int_type /= Void
	double_type_exists: double_type /= Void
	string_type_exists: string_type /= Void
	hashcode_type_exists: hashcode_type /= Void
	boolean_classes_not_empty: not boolean_classes.is_empty
	int_classes_not_empty: not int_classes.is_empty
	double_classes_not_empty: not double_classes.is_empty
	string_classes_not_empty: not string_classes.is_empty
	hashcode_classes_not_empty: not hashcode_classes.is_empty
	-- `hashcode_classes' is not empty in current implementation, but conceptually it can be,
	-- because hashcode type is intended to represent nonprintable types
end
