indexing
	description: "Kinds of types that are unknown during instrumentation"
	author: "Nadia Polikarpova"
	date: "$Date$"
	revision: "$Revision$"

class
	CI_UNKNOWN

inherit
	CI_VARIABLE_TYPE_KIND
		rename
			make as make_with_universe
		export {NONE}
			make_with_universe
		end

	CI_IDENTITIES_NAMES_CLIENT
create
	make

feature -- Initialization
	make (a_surrounding_class_name: like surrounding_class_name; a_universe: like universe) is
			-- Create unknown type kind for a variable that appears in `a_surrounding_class' within `a_universe'
		require
			a_surrounding_class_name_exists: a_surrounding_class_name /= Void
			a_universe_exists: a_universe /= Void
		do
			make_with_universe (a_universe)
			surrounding_class_name := a_surrounding_class_name
		ensure
			surrounding_class_name_set: surrounding_class_name = a_surrounding_class_name
			universe_set: universe = a_universe
		end

feature -- Access
	identity (a_variable: CI_VARIABLE): ET_EXPRESSION is
			-- Expression with printable result that identifies value of `a_variable' with type kind `Current'
		do
			create {ET_CALL_EXPRESSION} Result.make (
				create {ET_CALL_EXPRESSION}.make (Void, Identities_names.identity_target_name (surrounding_class_name), Void),
				Identities_names.unknown_identity_name,
				single_actual (a_variable.as_expression))
		end

	uniform_identity (an_expression: ET_IDENTIFIER): ET_EXPRESSION is
			-- Identity expression that has the same type (`INTEGER') independently of `base_type'
		do
			create {ET_CALL_EXPRESSION} Result.make (Void, Identities_names.unknown_identity_name, single_actual (an_expression))
		end

	uniform_identity_argument_type: ET_TYPE is
			-- Reference type, which is a valid "argument" type for `uniform_identity'
		once
			Result := universe.any_type
		end

	representation_type (a_class_name: ET_CLASS_NAME): STRING is
			-- Name of Daikon representation type for unknown type
		once
			Result := hashcode_type
		end

	surrounding_class_name: ET_CLASS_NAME
			-- Name of class, where variable is used (to be used in `identitiy' expression)
			-- Can be set to *UNKNOWN*, if `identity' expression is not used
invariant
	surrounding_class_name_exists: surrounding_class_name /= Void
end
